/*
    Copyright 2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef WIN32
    #error Windows only
#endif

#include <windows.h>
#include <direct.h>



#define VER     "0.1"



int its_hidden(char *folder, DWORD attr);
int dir_checks(char *in, char *dirname);
void recursive_dir(char *filedir);
void std_err(void);



int     counter = 0;



int main(int argc, char *argv[]) {
    char    cdir[MAX_PATH + 1];


    setbuf(stdout, NULL);

    fputs("\n"
        "Hide My Files folders unhider and scanner "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <folder>\n"
            "\n"
            "  Folder is the hidden folder (you need to activate the visualization of the\n"
            "  hidden files) or a normal folder or disk that contains the folders hidden\n"
            "  with the program \"Hide my files\".\n"
            "  If the provided folder is not hidden it will be scanned for searching all the\n"
            "  hidden folders contained in it.\n"
            "  The tool will ever ask you if you want to unhide each hidden folder\n"
            "\n", argv[0]);
        fputs("\n  Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    if(!its_hidden(argv[1], GetFileAttributes(argv[1]))) {
        dir_checks(argv[1], cdir);
        chdir(cdir);
        fputs("\n  start directory scanning:\n", stdout);
        recursive_dir(".");
    }
    fputs("\nFinished\n", stdout);
    fgetc(stdin);
    return(0);
}



int its_hidden(char *folder, DWORD attr) {
    int     i;
    const static char   bar[]  = "|/-\\";
    char    *ext[] = {
                "{00021401-0000-0000-C000-000000000046}",
                "{21EC2020-3AEA-1069-A2DD-08002B30309D}",
                "{2227A280-3AEA-1069-A2DE-08002B30309D}",
                "{25336920-03F9-11CF-8FD0-00AA00686F13}",
                "{645FF040-5081-101B-9F08-00AA002F954E}",
                "{D6277990-4C6A-11CF-8D87-00AA0060F5BF}",
                NULL
            },
            *p,
            *new;

    fputc(bar[counter & 3], stdout);    // counter
    fputc('\b', stdout);
    counter++;

    p = strrchr(folder, '.');
    if(!p) return(0);

    for(i = 0; ext[i]; i++) {
        if(!stricmp(p + 1, ext[i])) {
            *p = 0;
            new = strdup(folder);
            *p = '.';
            printf("\n"
                "  Hidden: %s\n"
                "       -> %s\n"
                "- Do you want to unhide this folder? (y/N)\n"
                "  ", folder, new);
            fflush(stdin);
            if((fgetc(stdin) | 0x20) == 'y') {
                fputs("- unhiding... ", stdout);
                if(rename(folder, new) < 0) std_err();
                if(attr & FILE_ATTRIBUTE_HIDDEN) {
                    SetFileAttributes(new, attr ^ FILE_ATTRIBUTE_HIDDEN);
                }
                fputs("ok\n", stdout);
            }
            free(new);
            return(1);
        }
    }
    return(0);
}



int dir_checks(char *in, char *dirname) {
    int     len;

    strcpy(dirname, in);

    len = strlen(dirname) - 1;
    if(len) {
        if(dirname[len] == ':') {   // root paths need backslash
            strcpy(dirname + len + 1, "\\");

        } else if(dirname[len - 1] == ':') {
            // nothing to do

        } else if((dirname[len] == '\\') || (dirname[len] == '/')) {
            dirname[len] = 0;
        }
    }

    printf("- check directory:    %s\n", dirname);

    if(chdir(dirname) < 0) std_err();
    getcwd(dirname, MAX_PATH);

    len = strlen(dirname) - 1;
    if((dirname[len] != '\\') && (dirname[len] != '/')) {
        len++;
        strcpy(dirname + len, "\\");
    }

    printf("                   -> %s\n", dirname);
    return(len + 1);
}



void recursive_dir(char *filedir) {
    char    tcDir[MAX_PATH + 1];
    WIN32_FIND_DATA wfd;
    HANDLE  hFind;
    int     plen;

    sprintf(tcDir, "%s\\%n*.*", filedir, &plen);

    hFind = FindFirstFile(tcDir, &wfd);
    if(!hFind) return;

    while(FindNextFile(hFind, &wfd)) {
        if(!strcmp(wfd.cFileName, "..")) continue;

        strcpy(tcDir + plen, wfd.cFileName);

        if(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
            its_hidden(tcDir, wfd.dwFileAttributes);
            recursive_dir(tcDir);
        }
    }
    FindClose(hFind);
}



void std_err(void) {
    perror("\nError");
    fputs("\n  Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


